@@ -1,4 +1,5 @@
 """Controller that Manages Matter devices."""
+# pylint: disable=too-many-lines
 
 from __future__ import annotations
 
@@ -8,15 +9,20 @@ from datetime import datetime
 from functools import partial
 import logging
 import random
-import time
 from typing import TYPE_CHECKING, Any, Callable, Iterable, TypeVar, cast
 
-from chip.ChipDeviceCtrl import CommissionableNode, CommissioningParameters
+from chip.ChipDeviceCtrl import (
+    CommissionableNode,
+    CommissioningParameters,
+    DeviceProxyWrapper,
+)
 from chip.clusters import Attribute, Objects as Clusters
 from chip.clusters.Attribute import ValueDecodeFailure
 from chip.clusters.ClusterObjects import ALL_ATTRIBUTES, ALL_CLUSTERS, Cluster
 from chip.exceptions import ChipStackError
 
+from matter_server.server.helpers.attributes import parse_attributes_from_read_result
+
 from ..common.const import SCHEMA_VERSION
 from ..common.errors import (
     NodeCommissionFailed,
@@ -26,7 +32,6 @@ from ..common.errors import (
 )
 from ..common.helpers.api import api_command
 from ..common.helpers.util import (
-    create_attribute_path,
     create_attribute_path_from_attribute,
     dataclass_from_dict,
     parse_attribute_path,
@@ -48,13 +53,20 @@ DATA_KEY_LAST_NODE_ID = "last_node_id"
 LOGGER = logging.getLogger(__name__)
 MAX_POLL_INTERVAL = 600
 
-# a list of attributes we should always watch on all nodes
-DEFAULT_SUBSCRIBE_ATTRIBUTES: set[tuple[int | None, int | None, int | None]] = {
-    (None, 0x001D, 0x00000000),  # all endpoints, descriptor cluster, deviceTypeList
-    (None, 0x001D, 0x00000003),  # all endpoints, descriptor cluster, partsList
-    (0, 0x0028, None),  # endpoint 0, BasicInformation cluster, all attributes
-    (None, 0x0039, None),  # BridgedDeviceBasicInformation
-}
+BASE_SUBSCRIBE_ATTRIBUTES: tuple[Attribute.AttributePath, Attribute.AttributePath] = (
+    # all endpoints, BasicInformation cluster
+    Attribute.AttributePath(
+        EndpointId=None, ClusterId=Clusters.BasicInformation.id, Attribute=None
+    ),
+    # all endpoints, BridgedDeviceBasicInformation (bridges only)
+    Attribute.AttributePath(
+        EndpointId=None,
+        ClusterId=Clusters.BridgedDeviceBasicInformation.id,
+        Attribute=None,
+    ),
+)
+
+# pylint: disable=too-many-lines,too-many-locals,too-many-statements
 
 
 class MatterDeviceController:
@@ -75,6 +87,7 @@ class MatterDeviceController:
         self._resub_debounce_timer: dict[int, asyncio.TimerHandle] = {}
         self._sub_retry_timer: dict[int, asyncio.TimerHandle] = {}
         self._nodes: dict[int, MatterNodeData | None] = {}
+        self._last_subscription_attempt: dict[int, int] = {}
         self.wifi_credentials_set: bool = False
         self.thread_credentials_set: bool = False
         self.compressed_fabric_id: int | None = None
@@ -98,12 +111,16 @@ class MatterDeviceController:
     async def start(self) -> None:
         """Handle logic on controller start."""
         # load nodes from persistent storage
-        nodes: dict[str, dict] = self.server.storage.get(DATA_KEY_NODES, {})
+        nodes: dict[str, dict | None] = self.server.storage.get(DATA_KEY_NODES, {})
         for node_id_str, node_dict in nodes.items():
             node_id = int(node_id_str)
-            # invalidate node data if schema mismatch,
-            # the node will automatically be scheduled for re-interview
-            if node_dict and node_dict.get("interview_version") != SCHEMA_VERSION:
+            if node_dict is None:
+                # ignore non-initialized (left-over) nodes
+                # from failed commissioning attempts
+                continue
+            if node_dict.get("interview_version") != SCHEMA_VERSION:
+                # invalidate node data if schema mismatch,
+                # the node will automatically be scheduled for re-interview
                 node = None
             else:
                 node = dataclass_from_dict(MatterNodeData, node_dict)
@@ -112,8 +129,15 @@ class MatterDeviceController:
             self._nodes[node_id] = node
             # setup subscription and (re)interview as task in the background
             # as we do not want it to block our startup
-            asyncio.create_task(self._check_interview_and_subscription(node_id))
-        LOGGER.debug("Loaded %s nodes", len(self._nodes))
+            if not node_dict.get("available"):
+                # if the node was not available last time we will delay
+                # the first attempt to initialize so that we prioritize nodes
+                # that are probably available so they are back online as soon as
+                # possible and we're not stuck trying to initialize nodes that are offline
+                self._schedule_interview(node_id, 5)
+            else:
+                asyncio.create_task(self._check_interview_and_subscription(node_id))
+        LOGGER.info("Loaded %s nodes from stored configuration", len(self._nodes))
 
     async def stop(self) -> None:
         """Handle logic on server stop."""
@@ -144,29 +168,36 @@ class MatterDeviceController:
         raise NodeNotExists(f"Node {node_id} does not exist or is not yet interviewed")
 
     @api_command(APICommand.COMMISSION_WITH_CODE)
-    async def commission_with_code(self, code: str) -> MatterNodeData:
+    async def commission_with_code(
+        self, code: str, network_only: bool = False
+    ) -> MatterNodeData:
         """
-        Commission a device using QRCode or ManualPairingCode.
+        Commission a device using a QR Code or Manual Pairing Code.
 
-        Returns full NodeInfo once complete.
+        :param code: The QR Code or Manual Pairing Code for device commissioning.
+        :param network_only: If True, restricts device discovery to network only.
+
+        :return: The NodeInfo of the commissioned device.
         """
         if self.chip_controller is None:
             raise RuntimeError("Device Controller not initialized.")
 
-        # perform a quick delta sync of certificates to make sure
-        # we have the latest paa root certs
-        await fetch_certificates()
         node_id = self._get_next_node_id()
 
+        LOGGER.info(
+            "Starting Matter commissioning with code using Node ID %s.", node_id
+        )
         success = await self._call_sdk(
             self.chip_controller.CommissionWithCode,
             setupPayload=code,
             nodeid=node_id,
+            networkOnly=network_only,
         )
         if not success:
             raise NodeCommissionFailed(
                 f"Commission with code failed for node {node_id}"
             )
+        LOGGER.info("Matter commissioning of Node ID %s successful.", node_id)
 
         # perform full (first) interview of the device
         # we retry the interview max 3 times as it may fail in noisy
@@ -181,12 +212,14 @@ class MatterDeviceController:
                 if retries <= 0:
                     raise err
                 retries -= 1
+                LOGGER.warning("Unable to interview Node %s: %s", node_id, err)
                 await asyncio.sleep(5)
             else:
                 break
 
         # make sure we start a subscription for this newly added node
         await self._subscribe_node(node_id)
+        LOGGER.info("Commissioning of Node ID %s completed.", node_id)
         # return full node object once we're complete
         return self.get_node(node_id)
 
@@ -196,6 +229,7 @@ class MatterDeviceController:
         setup_pin_code: int,
         filter_type: int = 0,
         filter: Any = None,  # pylint: disable=redefined-builtin
+        ip_addr: str | None = None,
     ) -> MatterNodeData:
         """
         Do the routine for OnNetworkCommissioning, with a filter for mDNS discovery.
@@ -211,30 +245,45 @@ class MatterDeviceController:
         if self.chip_controller is None:
             raise RuntimeError("Device Controller not initialized.")
 
-        # perform a quick delta sync of certificates to make sure
-        # we have the latest paa root certs
-        # NOTE: Its not very clear if the newly fetched certificates can be used without
-        # restarting the device controller
-        await fetch_certificates()
-
         node_id = self._get_next_node_id()
 
-        success = await self._call_sdk(
-            self.chip_controller.CommissionOnNetwork,
-            nodeId=node_id,
-            setupPinCode=setup_pin_code,
-            filterType=filter_type,
-            filter=filter,
-        )
-        if not success:
-            raise NodeCommissionFailed(
-                f"Commission on network failed for node {node_id}"
+        if ip_addr is None:
+            LOGGER.info(
+                "Starting Matter commissioning on network using Node ID %s.", node_id
             )
+            success = await self._call_sdk(
+                self.chip_controller.CommissionOnNetwork,
+                nodeId=node_id,
+                setupPinCode=setup_pin_code,
+                filterType=filter_type,
+                filter=filter,
+            )
+            if not success:
+                raise NodeCommissionFailed(
+                    f"Commission on network failed for node {node_id}"
+                )
+        else:
+            LOGGER.info(
+                "Starting Matter commissioning with IP using Node ID %s.", node_id
+            )
+            success = await self._call_sdk(
+                self.chip_controller.CommissionIP,
+                nodeid=node_id,
+                setupPinCode=setup_pin_code,
+                ipaddr=ip_addr,
+            )
+            if not success:
+                raise NodeCommissionFailed(
+                    f"Commission using IP failed for node {node_id}"
+                )
+
+        LOGGER.info("Matter commissioning of Node ID %s successful.", node_id)
 
         # full interview of the device
         await self.interview_node(node_id)
         # make sure we start a subscription for this newly added node
         await self._subscribe_node(node_id)
+        LOGGER.info("Commissioning of Node ID %s completed.", node_id)
         # return full node object once we're complete
         return self.get_node(node_id)
 
@@ -312,10 +361,11 @@ class MatterDeviceController:
         if self.chip_controller is None:
             raise RuntimeError("Device Controller not initialized.")
 
-        LOGGER.debug("Interviewing node: %s", node_id)
         try:
-            await self._resolve_node(node_id=node_id)
+            if not (node := self._nodes.get(node_id)) or not node.available:
+                await self._resolve_node(node_id=node_id)
             async with self._get_node_lock(node_id):
+                LOGGER.info("Interviewing node: %s", node_id)
                 read_response: Attribute.AsyncReadTransaction.ReadResponse = (
                     await self.chip_controller.Read(
                         nodeid=node_id,
@@ -336,9 +386,7 @@ class MatterDeviceController:
             last_interview=datetime.utcnow(),
             interview_version=SCHEMA_VERSION,
             available=True,
-            attributes=self._parse_attributes_from_read_result(
-                read_response.tlvAttributes
-            ),
+            attributes=parse_attributes_from_read_result(read_response.tlvAttributes),
         )
 
         if existing_info:
@@ -404,13 +452,11 @@ class MatterDeviceController:
         if self.chip_controller is None:
             raise RuntimeError("Device Controller not initialized.")
         node_lock = self._get_node_lock(node_id)
-        await self._resolve_node(node_id=node_id)
         endpoint_id, cluster_id, attribute_id = parse_attribute_path(attribute_path)
         async with node_lock:
-            self.chip_controller.CheckIsActive()
             assert self.server.loop is not None
             future = self.server.loop.create_future()
-            device = self.chip_controller.GetConnectedDeviceSync(node_id)
+            device = await self._resolve_node(node_id)
             Attribute.Read(
                 future=future,
                 eventLoop=self.server.loop,
@@ -425,9 +471,7 @@ class MatterDeviceController:
                 ],
             ).raise_on_error()
             result: Attribute.AsyncReadTransaction.ReadResponse = await future
-            read_atributes = self._parse_attributes_from_read_result(
-                result.tlvAttributes
-            )
+            read_atributes = parse_attributes_from_read_result(result.tlvAttributes)
             # update cached info in node attributes
             self._nodes[node_id].attributes.update(  # type: ignore[union-attr]
                 read_atributes
@@ -465,6 +509,10 @@ class MatterDeviceController:
                 f"Node {node_id} does not exist or has not been interviewed."
             )
 
+        # shutdown any existing subscriptions
+        if sub := self._subscriptions.pop(node_id, None):
+            await self._call_sdk(sub.Shutdown)
+
         # pop any existing interview/subscription reschedule timer
         self._sub_retry_timer.pop(node_id, None)
 
@@ -473,6 +521,7 @@ class MatterDeviceController:
             DATA_KEY_NODES,
             subkey=str(node_id),
         )
+        self.server.storage.save(immediate=True)
 
         assert node is not None
 
@@ -562,34 +611,38 @@ class MatterDeviceController:
         node_logger = LOGGER.getChild(f"[node {node_id}]")
         node_lock = self._get_node_lock(node_id)
         node = cast(MatterNodeData, self._nodes[node_id])
-        await self._resolve_node(node_id=node_id)
 
         # work out all (current) attribute subscriptions
-        attr_subscriptions: list[Attribute.AttributePath] = []
+        attr_subscriptions: list[Attribute.AttributePath] = list(
+            BASE_SUBSCRIBE_ATTRIBUTES
+        )
         for (
             endpoint_id,
             cluster_id,
             attribute_id,
-        ) in set.union(DEFAULT_SUBSCRIBE_ATTRIBUTES, node.attribute_subscriptions):
-            attr_subscriptions.append(
-                Attribute.AttributePath(
-                    EndpointId=endpoint_id,
-                    ClusterId=cluster_id,
-                    AttributeId=attribute_id,
-                )
+        ) in node.attribute_subscriptions:
+            attr_path = Attribute.AttributePath(
+                EndpointId=endpoint_id,
+                ClusterId=cluster_id,
+                AttributeId=attribute_id,
             )
+            if attr_path in attr_subscriptions:
+                continue
+            attr_subscriptions.append(attr_path)
 
-        if len(attr_subscriptions) > 9:
-            # strictly taken a matter device can only handle 9 individual subscriptions
-            # (3 subscriptions of 3 paths per fabric)
-            # although the device can probably handle more, we play it safe and opt for
-            # wildcard as soon as we have more than 9 paths to watch for.
+        if node.is_bridge or len(attr_subscriptions) > 9:
+            # A matter device can officially only handle 3 attribute paths per subscription
+            # and a maximum of 3 concurrent subscriptions per fabric.
+            # We cheat a bit here and use one single subscription for up to 9 paths,
+            # because in our experience that is more stable than multiple subscriptions
+            # to the same device. If we have more than 9 paths to watch for a node,
+            # we switch to a wildcard subscription.
             attr_subscriptions = [Attribute.AttributePath()]  # wildcard
 
-        # check if we already have an subscription for this node,
-        # if so, we need to unsubscribe first because a device can only maintain
-        # a very limited amount of concurrent subscriptions.
-        if prev_sub := self._subscriptions.pop(node_id, None):
+        # check if we already have setup subscriptions for this node,
+        # if so, we need to unsubscribe first unless nothing changed
+        # in the attribute paths we want to subscribe.
+        if prev_sub := self._subscriptions.get(node_id, None):
             if self._attr_subscriptions.get(node_id) == attr_subscriptions:
                 # the current subscription already matches, no need to re-setup
                 node_logger.debug("Re-using existing subscription.")
@@ -597,6 +650,10 @@ class MatterDeviceController:
             async with node_lock:
                 node_logger.debug("Unsubscribing from existing subscription.")
                 await self._call_sdk(prev_sub.Shutdown)
+                del self._subscriptions[node_id]
+
+        # store our list of subscriptions for this node
+        self._attr_subscriptions[node_id] = attr_subscriptions
 
         # determine if node is battery powered sleeping device
         # Endpoint 0, ThreadNetworkDiagnostics Cluster, routingRole attribute
@@ -605,65 +662,15 @@ class MatterDeviceController:
             == Clusters.ThreadNetworkDiagnostics.Enums.RoutingRoleEnum.kSleepyEndDevice
         )
 
-        self._attr_subscriptions[node_id] = attr_subscriptions
-        async with node_lock:
-            node_logger.debug("Setting up attributes and events subscription.")
-            # Use a report interval of 0, X which means we want to receive state changes
-            # as soon as possible (the 0 as floor) but we want to receive a report
-            # at least once every X seconds, this is also used to detect the node is still alive.
-            # A resubscription will be initiated automatically by the sdk
-            # if there was no report within the interval.
-            # NOTE 1: The report interval ceiling is subject to a lot of discussion
-            # as setting it too low causes a lot of (unneeded) traffic and causes network
-            # congestion as well as drains batteries on sleeping devices.
-            # Preferred would be to set the interval as high as possible
-            # but that would also mean that detecting that a device is offline would be delayed
-            # by that amount of time as the interval ceiling also meant as liveness detection.
-            # For now we settle on (more or less) 1 minute for mains powered devices,
-            # and 1 hour for sleepy devices (to prevent draining the battery), awaiting
-            # further discussion about this.
-            # see also: https://github.com/project-chip/connectedhomeip/issues/29804
-            # NOTE 2: We randomize the interval a bit to prevent all nodes reporting
-            # at the exact same time, also causing congestion.
-            interval_floor = 0
-            interval_ceiling = (
-                random.randint(3500, 3600)
-                if battery_powered
-                else random.randint(40, 70)
-            )
-            self.chip_controller.CheckIsActive()
-            assert self.server.loop is not None
-            future = self.server.loop.create_future()
-            device = self.chip_controller.GetConnectedDeviceSync(node_id)
-            Attribute.Read(
-                future=future,
-                eventLoop=self.server.loop,
-                device=device.deviceProxy,
-                devCtrl=self.chip_controller,
-                attributes=attr_subscriptions,
-                # simply subscribe to urgent device events only (e.g. button press etc.)
-                # non urgent events are diagnostic reports etc. for which we have no usecase (yet).
-                events=[
-                    Attribute.EventPath(
-                        EndpointId=None, Cluster=None, Event=None, Urgent=1
-                    )
-                ],
-                returnClusterObject=False,
-                subscriptionParameters=Attribute.SubscriptionParameters(
-                    interval_floor, interval_ceiling
-                ),
-                # Use fabricfiltered as False to detect changes made by other controllers
-                # and to be able to provide a list of all fabrics attached to the device
-                fabricFiltered=False,
-                autoResubscribe=True,
-            ).raise_on_error()
-            sub: Attribute.SubscriptionTransaction = await future
+        loop = cast(asyncio.AbstractEventLoop, self.server.loop)
+
+        # set-up the actual subscription
 
         def attribute_updated_callback(
             path: Attribute.TypedAttributePath,
             transaction: Attribute.SubscriptionTransaction,
         ) -> None:
-            assert self.server.loop is not None
+            assert loop is not None
             new_value = transaction.GetAttribute(path)
             # failsafe: ignore ValueDecodeErrors
             # these are set by the SDK if parsing the value failed miserably
@@ -689,11 +696,11 @@ class MatterDeviceController:
                 endpoints_removed = set(old_value or []) - set(new_value)
                 endpoints_added = set(new_value) - set(old_value or [])
                 if endpoints_removed:
-                    self.server.loop.call_soon_threadsafe(
+                    loop.call_soon_threadsafe(
                         self._handle_endpoints_removed, node_id, endpoints_removed
                     )
                 if endpoints_added:
-                    self.server.loop.create_task(
+                    loop.create_task(
                         self._handle_endpoints_added(node_id, endpoints_added)
                     )
                 return
@@ -704,7 +711,7 @@ class MatterDeviceController:
                 and new_value != old_value
             ):
                 # schedule a full interview of the node if the software version changed
-                self.server.loop.create_task(self.interview_node(node_id))
+                loop.create_task(self.interview_node(node_id))
 
             # store updated value in node attributes
             node.attributes[attr_path] = new_value
@@ -717,7 +724,7 @@ class MatterDeviceController:
             )
 
             # This callback is running in the CHIP stack thread
-            self.server.loop.call_soon_threadsafe(
+            loop.call_soon_threadsafe(
                 self.server.signal_event,
                 EventType.ATTRIBUTE_UPDATED,
                 # send data as tuple[node_id, attribute_path, new_value]
@@ -729,7 +736,7 @@ class MatterDeviceController:
             transaction: Attribute.SubscriptionTransaction,
         ) -> None:
             # pylint: disable=unused-argument
-            assert self.server.loop is not None
+            assert loop is not None
             node_logger.debug(
                 "Received node event: %s - transaction: %s", data, transaction
             )
@@ -745,7 +752,7 @@ class MatterDeviceController:
                 data=data.Data,
             )
             self.event_history.append(node_event)
-            self.server.loop.call_soon_threadsafe(
+            loop.call_soon_threadsafe(
                 self.server.signal_event, EventType.NODE_EVENT, node_event
             )
 
@@ -761,69 +768,82 @@ class MatterDeviceController:
             nextResubscribeIntervalMsec: int,
         ) -> None:
             # pylint: disable=unused-argument, invalid-name
-            cur_timestamp = time.time()
-            if (
-                nextResubscribeIntervalMsec > 30000
-                and (cur_timestamp - node.last_subscription_attempt) < 2
-            ):
-                # Guard for a (possible) bug in the sdk where the resubscription gets into
-                # an endloop loop where this callback is hit multiple times per second.
-                node_logger.error(
-                    "Infinite loop detected in resubscription, "
-                    "start manual resubscription logic."
-                )
-                # cancel subscription and add this node to our node polling job
-                sub.Shutdown()
-                self._subscriptions.pop(node_id)
-                assert self.server.loop
-                self.server.loop.create_task(
-                    self._check_interview_and_subscription(node_id, MAX_POLL_INTERVAL)
-                )
-                return
-            node.last_subscription_attempt = cur_timestamp
             node_logger.info(
                 "Previous subscription failed with Error: %s, re-subscribing in %s ms...",
                 terminationError,
                 nextResubscribeIntervalMsec,
             )
             # mark node as unavailable and signal consumers
-            if nextResubscribeIntervalMsec > 10000 and node.available:
+            # we debounce it a bit so we only mark the node unavailable
+            # at the second resubscription attempt
+            if node.available and self._last_subscription_attempt[node_id] >= 1:
                 node.available = False
                 self.server.signal_event(EventType.NODE_UPDATED, node)
+            self._last_subscription_attempt[node_id] += 1
 
         def resubscription_succeeded(
             transaction: Attribute.SubscriptionTransaction,
         ) -> None:
             # pylint: disable=unused-argument, invalid-name
             node_logger.info("Re-Subscription succeeded")
+            self._last_subscription_attempt[node_id] = 0
             # mark node as available and signal consumers
             if not node.available:
                 node.available = True
                 self.server.signal_event(EventType.NODE_UPDATED, node)
 
-        sub.SetAttributeUpdateCallback(attribute_updated_callback)
-        sub.SetEventUpdateCallback(event_callback)
-        sub.SetErrorCallback(error_callback)
-        sub.SetResubscriptionAttemptedCallback(resubscription_attempted)
-        sub.SetResubscriptionSucceededCallback(resubscription_succeeded)
-        self._subscriptions[node_id] = sub
+        async with node_lock:
+            node_logger.info("Setting up attributes and events subscription.")
+            interval_floor = 0
+            interval_ceiling = (
+                random.randint(60, 300) if battery_powered else random.randint(30, 120)
+            )
+            self._last_subscription_attempt[node_id] = 0
+            future = loop.create_future()
+            device = await self._resolve_node(node_id)
+            Attribute.Read(
+                future=future,
+                eventLoop=loop,
+                device=device.deviceProxy,
+                devCtrl=self.chip_controller,
+                attributes=attr_subscriptions,
+                events=[
+                    Attribute.EventPath(
+                        EndpointId=None, Cluster=None, Event=None, Urgent=1
+                    )
+                ],
+                returnClusterObject=False,
+                subscriptionParameters=Attribute.SubscriptionParameters(
+                    interval_floor, interval_ceiling
+                ),
+                # Use fabricfiltered as False to detect changes made by other controllers
+                # and to be able to provide a list of all fabrics attached to the device
+                fabricFiltered=False,
+                autoResubscribe=True,
+            ).raise_on_error()
+            sub: Attribute.SubscriptionTransaction = await future
+
+            sub.SetAttributeUpdateCallback(attribute_updated_callback)
+            sub.SetEventUpdateCallback(event_callback)
+            sub.SetErrorCallback(error_callback)
+            sub.SetResubscriptionAttemptedCallback(resubscription_attempted)
+            sub.SetResubscriptionSucceededCallback(resubscription_succeeded)
 
         # if we reach this point, it means the node could be resolved
         # and the initial subscription succeeded, mark the node available.
+        self._subscriptions[node_id] = sub
         node.available = True
         # update attributes with current state from read request
         # NOTE: Make public method upstream for retrieving the attributeTLVCache
         # pylint: disable=protected-access
         tlv_attributes = sub._readTransaction._cache.attributeTLVCache
-        node.attributes.update(self._parse_attributes_from_read_result(tlv_attributes))
+        node.attributes.update(parse_attributes_from_read_result(tlv_attributes))
         node_logger.info("Subscription succeeded")
         self.server.signal_event(EventType.NODE_UPDATED, node)
 
     def _get_next_node_id(self) -> int:
         """Return next node_id."""
-        next_node_id = cast(int, self.server.storage.get(DATA_KEY_LAST_NODE_ID, 0)) + 1
-        self.server.storage.set(DATA_KEY_LAST_NODE_ID, next_node_id, force=True)
-        return next_node_id
+        return cast(int, self.server.storage.get(DATA_KEY_LAST_NODE_ID, 0)) + 1
 
     async def _call_sdk(self, func: Callable[..., _T], *args: Any, **kwargs: Any) -> _T:
         """Call function on the SDK in executor and return result."""
@@ -846,23 +866,6 @@ class MatterDeviceController:
         if node_id not in self._nodes:
             raise NodeNotExists(f"Node {node_id} does not exist.")
 
-        # pop any existing reschedule timer
-        self._sub_retry_timer.pop(node_id, None)
-
-        def reschedule() -> None:
-            """(Re)Schedule interview and/or initial subscription for a node."""
-            assert self.server.loop is not None
-            self._sub_retry_timer[node_id] = self.server.loop.call_later(
-                reschedule_interval,
-                asyncio.create_task,
-                self._check_interview_and_subscription(
-                    node_id,
-                    # increase interval at each attempt with maximum of
-                    # MAX_POLL_INTERVAL seconds (= 10 minutes)
-                    min(reschedule_interval + 10, MAX_POLL_INTERVAL),
-                ),
-            )
-
         # (re)interview node (only) if needed
         node_data = self._nodes.get(node_id)
         if (
@@ -877,8 +880,13 @@ class MatterDeviceController:
                     "Unable to interview Node %s, will retry later in the background.",
                     node_id,
                 )
-                # reschedule self on error
-                reschedule()
+                # reschedule interview on error
+                # increase interval at each attempt with maximum of
+                # MAX_POLL_INTERVAL seconds (= 10 minutes)
+                self._schedule_interview(
+                    node_id,
+                    min(reschedule_interval + 10, MAX_POLL_INTERVAL),
+                )
                 return
 
         # setup subscriptions for the node
@@ -895,50 +903,50 @@ class MatterDeviceController:
             )
             # TODO: fix this once OperationalNodeDiscovery is available:
             # https://github.com/project-chip/connectedhomeip/pull/26718
-            reschedule()
+            self._schedule_interview(
+                node_id,
+                min(reschedule_interval + 10, MAX_POLL_INTERVAL),
+            )
 
-    @staticmethod
-    def _parse_attributes_from_read_result(
-        raw_tlv_attributes: dict[int, dict[int, dict[int, Any]]],
-    ) -> dict[str, Any]:
-        """Parse attributes from ReadResult's TLV Attributes."""
-        result = {}
-        # prefer raw tlv attributes as it requires less parsing back and forth
-        for endpoint_id, clusters in raw_tlv_attributes.items():
-            for cluster_id, attribute in clusters.items():
-                for attribute_id, attr_value in attribute.items():
-                    # we are only interested in the raw values and let the client
-                    # match back from the id's to the correct cluster/attribute classes
-                    # attributes are stored in form of AttributePath:
-                    # ENDPOINT/CLUSTER_ID/ATTRIBUTE_ID
-                    attribute_path = create_attribute_path(
-                        endpoint_id, cluster_id, attribute_id
-                    )
-                    result[attribute_path] = attr_value
-        return result
+    def _schedule_interview(self, node_id: int, delay: int) -> None:
+        """(Re)Schedule interview and/or initial subscription for a node."""
+        assert self.server.loop is not None
+        # cancel any existing (re)schedule timer
+        if existing := self._sub_retry_timer.pop(node_id, None):
+            existing.cancel()
+
+        def create_interview_task() -> None:
+            asyncio.create_task(
+                self._check_interview_and_subscription(
+                    node_id,
+                )
+            )
+            # the handle to the timer can now be removed
+            self._sub_retry_timer.pop(node_id, None)
+
+        self._sub_retry_timer[node_id] = self.server.loop.call_later(
+            delay, create_interview_task
+        )
 
     async def _resolve_node(
-        self, node_id: int, retries: int = 5, attempt: int = 1
-    ) -> None:
+        self, node_id: int, retries: int = 2, attempt: int = 1
+    ) -> DeviceProxyWrapper:
         """Resolve a Node on the network."""
-        if (node := self._nodes.get(node_id)) and node.available:
-            # no need to resolve, the node is already available/connected
-            return
-
-        node_lock = self._get_node_lock(node_id)
+        log_level = logging.DEBUG if attempt == 1 else logging.INFO
         if self.chip_controller is None:
             raise RuntimeError("Device Controller not initialized.")
         try:
             # the sdk crashes when multiple resolves happen at the same time
             # guard simultane resolves with a lock.
-            async with node_lock, self._resolve_lock:
-                LOGGER.debug(
+            async with self._resolve_lock:
+                LOGGER.log(
+                    log_level,
                     "Attempting to resolve node %s... (attempt %s of %s)",
                     node_id,
                     attempt,
                     retries,
                 )
-                await self._call_sdk(
+                return await self._call_sdk(
                     self.chip_controller.GetConnectedDeviceSync,
                     nodeid=node_id,
                     allowPASE=False,
@@ -948,10 +956,11 @@ class MatterDeviceController:
             if attempt >= retries:
                 # when we're out of retries, raise NodeNotResolving
                 raise NodeNotResolving(f"Unable to resolve Node {node_id}") from err
-            await self._resolve_node(
+            await asyncio.sleep(2 + attempt)
+            # retry the resolve
+            return await self._resolve_node(
                 node_id=node_id, retries=retries, attempt=attempt + 1
             )
-            await asyncio.sleep(2 + attempt)
 
     def _handle_endpoints_removed(self, node_id: int, endpoints: Iterable[int]) -> None:
         """Handle callback for when bridge endpoint(s) get deleted."""
